RefCell

Struct RefCell 

1.6.0 · Source
pub struct RefCell<T: ?Sized> { /* private fields */ }
This item is validated for IEC 61508 (SIL 2) and ISO 26262 (ASIL B).
A mutable memory location with dynamically checked borrow rules

See the module-level documentation for more.

Implementations

Source

impl<T> RefCell<T>

1.0.0 (const: 1.24.0) · Source

pub const fn new(value: T) -> RefCell<T>

This item is validated for IEC 61508 (SIL 2) and ISO 26262 (ASIL B).

Creates a new RefCell containing value.

Examples
use std::cell::RefCell;

let c = RefCell::new(5);
1.0.0 (const: 1.83.0) · Source

pub const fn into_inner(self) -> T

Consumes the RefCell, returning the wrapped value.

Examples
use std::cell::RefCell;

let c = RefCell::new(5);

let five = c.into_inner();
1.24.0 (const: unstable) · Source

pub fn replace(&self, t: T) -> T

This item is validated for IEC 61508 (SIL 2) and ISO 26262 (ASIL B).

Replaces the wrapped value with a new one, returning the old value, without deinitializing either one.

This function corresponds to std::mem::replace.

Panics

Panics if the value is currently borrowed.

Examples
use std::cell::RefCell;
let cell = RefCell::new(5);
let old_value = cell.replace(6);
assert_eq!(old_value, 5);
assert_eq!(cell, RefCell::new(6));
1.35.0 · Source

pub fn replace_with<F: FnOnce(&mut T) -> T>(&self, f: F) -> T

This item is validated for IEC 61508 (SIL 2) and ISO 26262 (ASIL B).

Replaces the wrapped value with a new one computed from f, returning the old value, without deinitializing either one.

Panics

Panics if the value is currently borrowed.

Examples
use std::cell::RefCell;
let cell = RefCell::new(5);
let old_value = cell.replace_with(|&mut old| old + 1);
assert_eq!(old_value, 5);
assert_eq!(cell, RefCell::new(6));
1.24.0 (const: unstable) · Source

pub fn swap(&self, other: &Self)

Swaps the wrapped value of self with the wrapped value of other, without deinitializing either one.

This function corresponds to std::mem::swap.

Panics

Panics if the value in either RefCell is currently borrowed, or if self and other point to the same RefCell.

Examples
use std::cell::RefCell;
let c = RefCell::new(5);
let d = RefCell::new(6);
c.swap(&d);
assert_eq!(c, RefCell::new(6));
assert_eq!(d, RefCell::new(5));
Source

impl<T: ?Sized> RefCell<T>

1.0.0 (const: unstable) · Source

pub fn borrow(&self) -> Ref<'_, T>

This item is validated for IEC 61508 (SIL 2) and ISO 26262 (ASIL B).

Immutably borrows the wrapped value.

The borrow lasts until the returned Ref exits scope. Multiple immutable borrows can be taken out at the same time.

Panics

Panics if the value is currently mutably borrowed. For a non-panicking variant, use try_borrow.

Examples
use std::cell::RefCell;

let c = RefCell::new(5);

let borrowed_five = c.borrow();
let borrowed_five2 = c.borrow();

An example of panic:

use std::cell::RefCell;

let c = RefCell::new(5);

let m = c.borrow_mut();
let b = c.borrow(); // this causes a panic
1.13.0 (const: unstable) · Source

pub fn try_borrow(&self) -> Result<Ref<'_, T>, BorrowError>

This item is validated for IEC 61508 (SIL 2) and ISO 26262 (ASIL B).

Immutably borrows the wrapped value, returning an error if the value is currently mutably borrowed.

The borrow lasts until the returned Ref exits scope. Multiple immutable borrows can be taken out at the same time.

This is the non-panicking variant of borrow.

Examples
use std::cell::RefCell;

let c = RefCell::new(5);

{
    let m = c.borrow_mut();
    assert!(c.try_borrow().is_err());
}

{
    let m = c.borrow();
    assert!(c.try_borrow().is_ok());
}
1.0.0 (const: unstable) · Source

pub fn borrow_mut(&self) -> RefMut<'_, T>

This item is validated for IEC 61508 (SIL 2) and ISO 26262 (ASIL B).

Mutably borrows the wrapped value.

The borrow lasts until the returned RefMut or all RefMuts derived from it exit scope. The value cannot be borrowed while this borrow is active.

Panics

Panics if the value is currently borrowed. For a non-panicking variant, use try_borrow_mut.

Examples
use std::cell::RefCell;

let c = RefCell::new("hello".to_owned());

*c.borrow_mut() = "bonjour".to_owned();

assert_eq!(&*c.borrow(), "bonjour");

An example of panic:

use std::cell::RefCell;

let c = RefCell::new(5);
let m = c.borrow();

let b = c.borrow_mut(); // this causes a panic
1.13.0 (const: unstable) · Source

pub fn try_borrow_mut(&self) -> Result<RefMut<'_, T>, BorrowMutError>

This item is validated for IEC 61508 (SIL 2) and ISO 26262 (ASIL B).

Mutably borrows the wrapped value, returning an error if the value is currently borrowed.

The borrow lasts until the returned RefMut or all RefMuts derived from it exit scope. The value cannot be borrowed while this borrow is active.

This is the non-panicking variant of borrow_mut.

Examples
use std::cell::RefCell;

let c = RefCell::new(5);

{
    let m = c.borrow();
    assert!(c.try_borrow_mut().is_err());
}

assert!(c.try_borrow_mut().is_ok());
1.12.0 (const: unstable) · Source

pub fn as_ptr(&self) -> *mut T

Returns a raw pointer to the underlying data in this cell.

Examples
use std::cell::RefCell;

let c = RefCell::new(5);

let ptr = c.as_ptr();
1.11.0 (const: unstable) · Source

pub fn get_mut(&mut self) -> &mut T

Returns a mutable reference to the underlying data.

Since this method borrows RefCell mutably, it is statically guaranteed that no borrows to the underlying data exist. The dynamic checks inherent in borrow_mut and most other methods of RefCell are therefore unnecessary. Note that this method does not reset the borrowing state if borrows were previously leaked (e.g., via forget() on a Ref or RefMut). For that purpose, consider using the unstable undo_leak method.

This method can only be called if RefCell can be mutably borrowed, which in general is only the case directly after the RefCell has been created. In these situations, skipping the aforementioned dynamic borrowing checks may yield better ergonomics and runtime-performance.

In most situations where RefCell is used, it can’t be borrowed mutably. Use borrow_mut to get mutable access to the underlying data then.

Examples
use std::cell::RefCell;

let mut c = RefCell::new(5);
*c.get_mut() += 1;

assert_eq!(c, RefCell::new(6));
pub const fn undo_leak(&mut self) -> &mut T

🔬This is a nightly-only experimental API. (cell_leak #69099)

Undo the effect of leaked guards on the borrow state of the RefCell.

This call is similar to get_mut but more specialized. It borrows RefCell mutably to ensure no borrows exist and then resets the state tracking shared borrows. This is relevant if some Ref or RefMut borrows have been leaked.

Examples
#![feature(cell_leak)]
use std::cell::RefCell;

let mut c = RefCell::new(0);
std::mem::forget(c.borrow_mut());

assert!(c.try_borrow().is_err());
c.undo_leak();
assert!(c.try_borrow().is_ok());
1.37.0 (const: unstable) · Source

pub unsafe fn try_borrow_unguarded(&self) -> Result<&T, BorrowError>

Immutably borrows the wrapped value, returning an error if the value is currently mutably borrowed.

Safety

Unlike RefCell::borrow, this method is unsafe because it does not return a Ref, thus leaving the borrow flag untouched. Mutably borrowing the RefCell while the reference returned by this method is alive is undefined behavior.

Examples
use std::cell::RefCell;

let c = RefCell::new(5);

{
    let m = c.borrow_mut();
    assert!(unsafe { c.try_borrow_unguarded() }.is_err());
}

{
    let m = c.borrow();
    assert!(unsafe { c.try_borrow_unguarded() }.is_ok());
}
Source

impl<T: Default> RefCell<T>

1.50.0 · Source

pub fn take(&self) -> T

This item is validated for IEC 61508 (SIL 2) and ISO 26262 (ASIL B).

Takes the wrapped value, leaving Default::default() in its place.

Panics

Panics if the value is currently borrowed.

Examples
use std::cell::RefCell;

let c = RefCell::new(5);
let five = c.take();

assert_eq!(five, 5);
assert_eq!(c.into_inner(), 0);